@@ -48,10 +48,12 @@
    (lambda ($1 . $rest) (make-tl 'expr-list $1))
    ;; argument-expression-list => argument-expression-list "," assignment-e...
    (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
-   ;; argument-expression-list => typedef-name
-   (lambda ($1 . $rest) (make-tl 'expr-list $1))
-   ;; argument-expression-list => argument-expression-list "," typedef-name
-   (lambda ($3 $2 $1 . $rest) (tl-append $1 $3))
+   ;; argument-expression-list => declaration-specifiers abstract-declarator
+   (lambda ($2 $1 . $rest)
+     (make-tl 'expr-list `(decl ,(tl->list $1) ,$2)))
+   ;; argument-expression-list => argument-expression-list "," declaration-...
+   (lambda ($4 $3 $2 $1 . $rest)
+     (tl-append $1 `(decl ,(tl->list $1) ,$2)))
    ;; unary-expression => postfix-expression
    (lambda ($1 . $rest) $1)
    ;; unary-expression => "++" unary-expression
@@ -628,8 +630,8 @@
    (lambda ($4 $3 $2 $1 . $rest) $1)
    ;; labeled-statement => "default" ":" statement
    (lambda ($3 $2 $1 . $rest) $1)
-   ;; compound-statement => "{" block-item-list "}" opt-code-comment
-   (lambda ($4 $3 $2 $1 . $rest)
+   ;; compound-statement => "{" block-item-list "}"
+   (lambda ($3 $2 $1 . $rest)
      `(compd-stmt ,(tl->list $2)))
    ;; compound-statement => "{" "}"
    (lambda ($2 $1 . $rest)
