@@ -20,10 +20,8 @@
 (define-module (nyacc lang c99 cpp)
   #:export (parse-cpp-line
 	    eval-cpp-expr
-	    ;;
-	    n-parse-cpp-expr
+	    parse-cpp-expr
 	    )
-  ;;#:use-module (nyacc lalr)
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
   #:use-module (nyacc lang util)
@@ -108,6 +106,8 @@ todo:
 	   ((include) (cpp-include))
 	   ((ifdef) `(if (defined ,(rd-ident))))
 	   ((ifndef) `(if (not (defined ,(rd-ident)))))
+	   ;;((ifdef) `(ifdef ,(rd-ident)))
+	   ;;((ifndef) `(ifndef ,(rd-ident)))
 	   ((define) (cpp-define))
 	   ((if elif) (list cmd (parse-cpp-expr)))
 	   ((else endif) (list cmd))
